<?php

namespace Service\Integration;

use Doctrine\ORM\EntityManager;
use Exception\CustomException;
use Util\Messages;
use Util\Utilities;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

class MovideskIntegration {

    const CLIENT_ID = "CustomerPortal";
    const CLIENT_SECRET = "8c673f38ade42e5e3318c830d9522bd2";
    const ROUTE_LINK_GET_TICKETS = '/public/v1/tickets?token=';

    protected $entityManager;
    protected $token;
    protected $MOVIDESK__URL;
    protected $environment;


    public function __construct(EntityManager $entityManager, \Util\Utilities $environment, $MOVIDESK__URL,$token){
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->MOVIDESK__URL = $MOVIDESK__URL;
        $this->token = $token;
    }

    public function findTicketById($idTicket){

        $url = $this->MOVIDESK__URL.self::ROUTE_LINK_GET_TICKETS.$this->token."&id=".$idTicket;
        return $this->requestMovdesk($url);
    }

    public function validateTicket($idTicket){
        $validate = false;
        $result = $this->findTicketById($idTicket);
        if(!empty($result)){
            $validate = $result['id'] == $idTicket;
        }
        return $validate;
    }

    public function findTicketMovDeskDeploy(){
        $filterServiceFirstLevelId = Utilities::getMovideskFilterServiceDeploy();
        $arrayfilter = explode(',',$filterServiceFirstLevelId);
        $urlFilter = $this->getStringFilterIn($arrayfilter,'serviceFirstLevelId');
        $url = $this->MOVIDESK__URL.self::ROUTE_LINK_GET_TICKETS.$this->token.
            '&$select=id,subject,serviceFirstLevelId'.$urlFilter.'%20and%20status%20ne%20%27Fechado%27%20and%20status%20ne%20%27Resolvido%27%20and%20status%20ne%20%27Cancelado%27';
        return $this->requestMovdesk($url);
    }

    /**
     * @param $url
     * @return bool|mixed|string
     */
    protected function requestMovdesk($url)
    {
        $header = array();
        $header[] = 'Content-Type: application/json';
        $curl_session = curl_init();

        curl_setopt($curl_session, CURLOPT_HTTPHEADER, $header);
        curl_setopt($curl_session, CURLOPT_URL, $url);
        curl_setopt($curl_session, CURLOPT_TIMEOUT, 86400);
        curl_setopt($curl_session, CURLOPT_RETURNTRANSFER, true);
        $result = curl_exec($curl_session);
        curl_close($curl_session);
        $result = json_decode($result, true);
        return $result;
    }

    public function createInternalAction($idTicket, $actionData)
    {
        $url = $this->MOVIDESK__URL.self::ROUTE_LINK_GET_TICKETS.$this->token."&id=".$idTicket;
        return $this->requestMovdeskPatch($url, $actionData);
    }

    /**
     * @param string $url
     * @param array $data
     * @return array
     */
    protected function requestMovdeskPatch($url, $data)
    {
        $headers = [
            'Content-Type: application/json',
        ];

        $curl = curl_init();

        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'PATCH'); // Define o método HTTP como PATCH
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($data));
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_TIMEOUT, 86400);

        $result = curl_exec($curl);
        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        $curlError = curl_error($curl);

        curl_close($curl);

        // Verifica se houve erro na execução do cURL
        if ($curlError) {
            return [
                'error' => true,
                'statusCode' => 0,
                'curlError' => $curlError,
            ];
        }

        // Retorna resposta formatada
        if ($httpCode >= 400) {
            return [
                'error' => true,
                'statusCode' => $httpCode,
                'response' => $result,
            ];
        }

        return [
            'error' => false,
            'statusCode' => $httpCode,
            'response' => json_decode($result, true),
        ];
    }

    protected function getStringFilterIn($ids,$field = 'id')
    {
        $urlFilter = '';
        foreach ($ids as $id) {
            if (!empty($urlFilter)) {
                $urlFilter .= '%20or%20';
            } else {
                $urlFilter = '&$filter=(';
            }
            $urlFilter .= "$field%20eq%20$id";
        }
        $urlFilter .= ')';
        return $urlFilter;
    }

}
